/* RemoteLoggingSinkProvider.cs
 * Implements a sink provider for RemoteLoggingSink.
 *
 * Distributed with no license what-so-ever - feel free to do
 * whatever the hell you want with this :-)  I would appreciate it
 * if you dropped me a line to let me know if you've found this
 * useful though.
 *
 * Have fun!
 *
 * Tomer Gabel
 * http://www.tomergabel.com
 * tomer@tomergabel.com
 *
 */
using System.Collections;
using System.Runtime.Remoting.Channels;

namespace TomerGabel.RemoteLogging
{
	public class RemoteLoggingSinkProvider : IServerChannelSinkProvider
	{
		private IServerChannelSinkProvider m_next;

		/// <summary>
		/// Gets or sets the
		/// next sink provider in the channel sink provider chain.
		/// </summary>
		/// <value></value>
		/// <exception cref="T:System.Security.SecurityException">The immediate caller does not have infrastructure permission.</exception>
		public IServerChannelSinkProvider Next
		{
			get { return m_next; }
			set { m_next = value; }
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="RemoteLoggingSinkProvider"/> class.
		/// </summary>
		public RemoteLoggingSinkProvider()
		{
			RemoteLoggingServices.Track();
		}
		
		/// <summary>
		/// Initializes a new instance of the <see cref="RemoteLoggingSinkProvider"/> class.
		/// </summary>
		/// <param name="properties">Properties for this sink provider (ignored).</param>
		/// <param name="providerData">Provider data for this sink provider (ignored).</param>
		/// <remarks>Required for use in configuration files. Implicitly enables tracking.</remarks>
		public RemoteLoggingSinkProvider( IDictionary properties, ICollection providerData ) : this()
		{
		}

		/// <summary>
		/// Returns the channel data for the channel that the
		/// current sink is associated with.
		/// </summary>
		/// <param name="channelData">An <see cref="T:System.Runtime.Remoting.Channels.IChannelDataStore"/> object in which the channel data is to be returned.</param>
		/// <exception cref="T:System.Security.SecurityException">The immediate caller does not have infrastructure permission.</exception>
		public void GetChannelData( IChannelDataStore channelData )
		{
		}

		/// <summary>
		/// Creates a sink chain.
		/// </summary>
		/// <param name="channel">The channel for which to create the channel sink chain.</param>
		/// <returns>
		/// The first sink of the newly formed channel sink chain, or
		/// <see langword="null"/> , indicating that this provider will not or cannot provide
		/// a connection for this endpoint.
		/// </returns>
		/// <exception cref="T:System.Security.SecurityException">The immediate caller does not have infrastructure permission.</exception>
		public IServerChannelSink CreateSink( IChannelReceiver channel )
		{
			return new RemoteLoggingSink( m_next.CreateSink( channel ) );
		}
	}
}
