using System;
using System.Runtime.Remoting;
using TomerGabel.RemoteLogging;

namespace RemotingLoggingExample
{
	[LogRemoteCall]
	public class ImpCAOObject : MarshalByRefObject, ICAOObject
	{
		public void DoSomething()
		{
			Console.WriteLine( "Invocation from CAO object" );
		}
	}

	class ImpExample : MarshalByRefObject, IExample
	{
		public override object InitializeLifetimeService()
		{
			return null;
		}

		
		[LogRemoteCall]
		public ICAOObject SendMessage( string message )
		{
			Console.WriteLine( "Message from client: {0}", message );
			return new ImpCAOObject();
		}
	}
	
	class Driver : IRemoteLoggingConsumer
	{
		// ... 
		
		public void HandleRemoteCall( object remote, string method, object[] args )
		{
			Console.WriteLine( "Logging framework intercepted a remote call on object {0}, method {1}",
			                   remote.GetHashCode(), method );
			int i = 0;
			foreach ( object arg in args )
				Console.WriteLine( "\t{0}: {1}", i++, arg == null ? "null" : arg );
		}

		public static void Main( string[] args )
		{
			// Set up Remoting
			RemotingConfiguration.Configure( "Server.exe.config" );
			
			// Create an example marshalled object
			ImpExample s = new ImpExample();
			Console.WriteLine( "New stub implementation {0} created", s.GetHashCode() );
			RemotingServices.Marshal( s, "test" );
			
			// Register ourselves as a consumer
			RemoteLoggingServices.Register( new Driver() );
			
			Console.WriteLine( "Server up and running, type anything to quit" );
			Console.ReadLine();
		}
	}
}